@@ -49,7 +49,7 @@ class Receiver(object):
         self.__properties = dict(properties)
 
         self.__compatible_remotes = properties.get('compatible-remotes')
-
+        
         if self.__compatible_remotes:
             self.__compatible_remotes = [
                 name.strip() for name in
@@ -81,6 +81,9 @@ class Receiver(object):
     def __repr__(self):
         return '<Receiver: vendor=%s, product=%s>' % (self.__vendor, self.__product)
 
+    def __getitem__(self, key):
+        return self.__properties[key]
+
     # pylint: disable-msg=W0212
     vendor = property(lambda self: self.__vendor)
     vendor_id = property(lambda self: self.__vendor_id)
@@ -88,6 +91,7 @@ class Receiver(object):
     product = property(lambda self: self.__product)
     product_id = property(lambda self: self.__product_id)
 
+    device = property(lambda self: self.__properties.get('device'))
     device_nodes = property(lambda self: self.__properties.get('device-nodes'))
     kernel_module = property(lambda self: self.__properties.get('kernel-module'))
     lirc_driver = property(lambda self: self.__properties.get('lirc-driver', 'default'))
@@ -193,14 +197,35 @@ class Remote(object):
 
         mechanism.WriteRemoteConfiguration(self.configuration)
 
+    def __set_contributor(self, value):
+        '''
+        Updates the contributor property of this remote.
+        '''
+        
+        self.__contributor = value
+
+    def __set_vendor(self, value):
+        '''
+        Updates the vendor property of this remote.
+        '''
+        
+        self.__vendor = value
+
+    def __set_product(self, value):
+        '''
+        Updates the product property of this remote.
+        '''
+        
+        self.__product = value
+
     def __repr__(self):
         return '<Remote: %s>' % self.__name
 
     # pylint: disable-msg=W0212
     name = property(lambda self: self.__name)
-    product = property(lambda self: self.__product)
-    vendor = property(lambda self: self.__vendor)
-    contributor = property(lambda self: self.__contributor)
+    product = property(lambda self: self.__product, __set_product)
+    vendor = property(lambda self: self.__vendor, __set_vendor)
+    contributor = property(lambda self: self.__contributor, __set_contributor)
 
     configuration = property(__get_configuration)
 
@@ -329,12 +354,37 @@ class RemotesDatabase(object):
                             previous_remote.filename)
 
     def get(self, name, default=None):
-        '''
-        Looks up the remote matching name.
-        '''
+        '''Looks up the remote matching name.'''
 
         return self.__remotes.get(name, default)
 
+    def find(self, vendor, product, default=None):
+        '''Tries to find the specified remote control.'''
+
+        # Cope with Unknown vendors:
+        # We store 'Unknown' (not translated) in the configuration file in backend.WriteRemoteConfiguration() if vendor is None:
+        vendor_name = vendor;
+        if(vendor_name == 'Unknown'):
+          vendor_name = None
+
+        # Try looking at the remote.product
+        # (the combo lists product.name if remote.product is None):
+        for remote in self:
+            #print ("RemotesDatabase.find(): remote.vendor=%s, remote_product=%s\n" % (remote.vendor, remote.product))
+            if (remote.vendor == vendor_name and
+                remote.product == product):
+                return remote
+
+        # Try looking at the remote.name
+        # (the combo lists product.name if remote.product is None):
+        for remote in self:
+            #print ("RemotesDatabase.find(): remote.vendor=%s, remote_product=%s\n" % (remote.vendor, remote.product))
+            if (remote.vendor == vendor_name and
+                remote.name == product):
+                return remote
+
+        return default
+
     def __iter__(self):
         return iter(self.__remotes.values())
     def __len__(self):
@@ -789,61 +839,102 @@ class KeyListener(gobject.GObject):
         # pylint: disable-msg=E1002
         super(KeyListener, self).__init__()
 
+        self.__running = False
         self.__socket_name = socket_name
         self.__reconnect_source = 0
         self.__socket_source = 0
         self.__socket = None
+        self.__buffer = ''
 
     def start(self):
-        '''
-        Starts listening.
-        '''
+        '''Starts listening.'''
+
+        self.__running = True
 
         if not self.__connect():
             self.__reconnect()
 
+    def stop(self):
+        '''Stops listening.'''
+
+        self.__running = False
+        self.__disconnect()
+
+    def __on_io_event(self, fd, condition):
+        '''Handle I/O events on the lircd socket.'''
+
+        logging.info('I/O event on lirc socket %d: %d', fd, condition)
+
+        if condition & gobject.IO_IN:
+            logging.info('reading from lirc socket %d...', fd)
+            packet = self.__socket and self.__socket.recv(128)
+
+            logging.info('...%d bytes received.', len(packet))
+            self.__buffer += packet
+
+            while True:
+                eol = self.__buffer.find('\n')
+
+                if eol < 0:
+                    break
+
+                packet = self.__buffer[:eol]
+                self.__buffer = self.__buffer[eol + 1:]
+
+                logging.info('processing packet: %r', packet)
+
+                try:
+                    code, repeat, name, remote = packet.split()
+                    repeat = int(repeat, 16)
+                    code = long(code, 16)
+
+                    # pylint: disable-msg=E1101
+                    self.emit('key-pressed', remote, repeat, name, code)
+
+                # pylint: disable-msg=W0704
+                except ValueError:
+                    pass
+
+        if condition & gobject.IO_HUP:
+            if self.__running:
+                def restart():
+                    if self.__running:
+                        self.start()
+
+                    return False
+
+                gobject.timeout_add(1000, restart)
+
+            else:
+                self.__disconnect()
+
+            return False
+
+        return True
+
     def __connect(self):
-        '''
-        Connects to the lircd socket.
-        '''
+        '''Connects to the lircd socket.'''
+
+        self.__disconnect()
+
+        logging.info('trying to connect to lircd...')
 
-        logging.info('Trying to connect to lircd...')
         self.__socket = None
+        self.__buffer = ''
 
         try:
-            # pylint: disable-msg=W0613
-            def on_io(fd, condition):
-                '''
-                Handle I/O events on the lircd socket.
-                '''
-
-                if condition & gobject.IO_IN:
-                    packet = self.__socket and self.__socket.recv(128)
-
-                    if not packet:
-                        self.__reconnect()
-                        return False
-
-                    try:
-                        code, repeat, name, remote = packet.split()
-                        repeat = int(repeat, 16)
-                        code = long(code, 16)
-
-                        # pylint: disable-msg=E1101
-                        self.emit('key-pressed', remote, repeat, name, code)
-
-                    # pylint: disable-msg=W0704
-                    except ValueError:
-                        pass
-
-                return True
-
             self.__socket = socket(AF_UNIX)
-            self.__socket_source = gobject.io_add_watch(self.__socket.fileno(),
-                                                        gobject.IO_IN, on_io)
+
+            self.__socket_source = (
+                gobject.io_add_watch(self.__socket.fileno(),
+                                     gobject.IO_IN | gobject.IO_HUP,
+                                     self.__on_io_event))
 
             self.__socket.connect(self.__socket_name)
-            logging.info('Listening on %s...', self.__socket_name)
+
+            logging.info('listening on %s (fd=%d, source=%d)...',
+                         self.__socket_name, self.__socket.fileno(),
+                         self.__socket_source)
 
             # pylint: disable-msg=E1101
             self.emit('changed')
@@ -852,16 +943,16 @@ class KeyListener(gobject.GObject):
 
         except SocketError, ex:
             if ex[0] not in (errno.ENOENT, errno.ECONNREFUSED):
-                # TODO: pychecker says "gnome_lirc_properties/lirc.py:855: Object (ex) has no attribute (message)"
+                # pychecker says "Object (ex) has no attribute (message)", and that's not true.
                 logging.error('Cannot connect to %s: %s', self.__socket_name, ex.message)
 
             self.__disconnect()
             return False
 
     def __disconnect(self):
-        '''
-        Disconnects from the lircd socket.
-        '''
+        '''Disconnects from the lircd socket.'''
+
+        self.__buffer = ''
 
         if self.__socket_source:
             logging.info('disconnecting source: %d', self.__socket_source)
@@ -877,19 +968,13 @@ class KeyListener(gobject.GObject):
         self.emit('changed')
 
     def __reconnect(self):
-        '''
-        Tries to reconnects to the lircd socket.
-        '''
-
-        self.__disconnect()
+        '''Tries to reconnects to the lircd socket.'''
 
         if not self.__reconnect_source:
             self.__reconnect_source = gobject.timeout_add(5000, self._on_reconnect)
 
     def _on_reconnect(self):
-        '''
-        Called regularly when the key listener tries to reconnect.
-        '''
+        '''Called regularly when the key listener tries to reconnect.'''
 
         if self.__connect():
             self.__reconnect_source = 0
@@ -957,9 +1042,35 @@ def find_remote_config():
     '''Finds the location our customized lircd.conf file.'''
 
     if has_include_keyword():
-        return os.path.join(config.LIRC_CONFDIR, 'lircd.conf.gnome'), True
+        return config.LIRC_REMOTE_CONF, True
 
-    return os.path.join(config.LIRC_CONFDIR, 'lircd.conf'), False
+    return config.LIRC_DAEMON_CONF, False
+
+def check_hardware_settings(selected_remote):
+    '''Check if the hardware settings are sane.'''
+
+    remote_config, can_include = find_remote_config()
+
+    if can_include and not re.search(
+        r'^\s*include\s+%s\s*$' % re.escape(config.LIRC_REMOTE_CONF),
+        open(config.LIRC_DAEMON_CONF).read(), re.MULTILINE):
+        return False
+
+    parser = RemotesParser()
+    parser.read(open(remote_config))
+
+    for remote in parser.remotes:
+        if remote is None:
+            return False
+
+        if selected_remote is None:
+            return False;
+
+        if (remote.vendor == selected_remote.vendor and
+            remote.product == selected_remote.product):
+            return True
+
+    return False
 
 if '__main__' == __name__:
     print find_remote_config()
